@@ -14,7 +14,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
    xrdp: A Remote Desktop Protocol server.
-   Copyright (C) Jay Sorg 2004-2009
+   Copyright (C) Jay Sorg 2004-2010
 
    rdp layer
 
@@ -57,13 +57,16 @@ static tui8 g_unknown2[8] =
 static int APP_CC
 xrdp_rdp_read_config(struct xrdp_client_info* client_info)
 {
-  int index;
-  struct list* items;
-  struct list* values;
-  char* item;
-  char* value;
+  int index = 0;
+  struct list* items = (struct list *)NULL;
+  struct list* values = (struct list *)NULL;
+  char* item = (char *)NULL;
+  char* value = (char *)NULL;
   char cfg_file[256];
 
+  /* initialize (zero out) local variables: */
+  g_memset(cfg_file,0,sizeof(char) * 256);
+
   items = list_create();
   items->auto_free = 1;
   values = list_create();
@@ -114,6 +117,10 @@ xrdp_rdp_read_config(struct xrdp_client_info* client_info)
         client_info->channel_code = 1;
       }
     }
+    else if (g_strcasecmp(item, "max_bpp") == 0)
+    {
+      client_info->max_bpp = g_atoi(value);
+    }
   }
   list_delete(items);
   list_delete(values);
@@ -122,9 +129,9 @@ xrdp_rdp_read_config(struct xrdp_client_info* client_info)
 
 /*****************************************************************************/
 struct xrdp_rdp* APP_CC
-xrdp_rdp_create(struct xrdp_session* session, int sck)
+xrdp_rdp_create(struct xrdp_session* session, struct trans* trans)
 {
-  struct xrdp_rdp* self;
+  struct xrdp_rdp* self = (struct xrdp_rdp *)NULL;
 
   DEBUG(("in xrdp_rdp_create"));
   self = (struct xrdp_rdp*)g_malloc(sizeof(struct xrdp_rdp), 1);
@@ -133,7 +140,7 @@ xrdp_rdp_create(struct xrdp_session* session, int sck)
   /* read ini settings */
   xrdp_rdp_read_config(&self->client_info);
   /* create sec layer */
-  self->sec_layer = xrdp_sec_create(self, sck, self->client_info.crypt_level,
+  self->sec_layer = xrdp_sec_create(self, trans, self->client_info.crypt_level,
                                     self->client_info.channel_code);
   /* default 8 bit v1 color bitmap cache entries and size */
   self->client_info.cache1_entries = 600;
@@ -142,6 +149,7 @@ xrdp_rdp_create(struct xrdp_session* session, int sck)
   self->client_info.cache2_size = 1024;
   self->client_info.cache3_entries = 262;
   self->client_info.cache3_size = 4096;
+  g_write_ip_address(trans->sck, self->client_info.client_ip);  /* load client ip info */
   DEBUG(("out xrdp_rdp_create"));
   return self;
 }
@@ -187,10 +195,10 @@ xrdp_rdp_init_data(struct xrdp_rdp* self, struct stream* s)
 int APP_CC
 xrdp_rdp_recv(struct xrdp_rdp* self, struct stream* s, int* code)
 {
-  int error;
-  int len;
-  int pdu_code;
-  int chan;
+  int error = 0;
+  int len = 0;
+  int pdu_code = 0;
+  int chan = 0;
 
   DEBUG(("in xrdp_rdp_recv"));
   if (s->next_packet == 0 || s->next_packet >= s->end)
@@ -248,7 +256,7 @@ xrdp_rdp_recv(struct xrdp_rdp* self, struct stream* s, int* code)
 int APP_CC
 xrdp_rdp_send(struct xrdp_rdp* self, struct stream* s, int pdu_type)
 {
-  int len;
+  int len = 0;
 
   DEBUG(("in xrdp_rdp_send"));
   s_pop_layer(s, rdp_hdr);
@@ -270,7 +278,7 @@ int APP_CC
 xrdp_rdp_send_data(struct xrdp_rdp* self, struct stream* s,
                    int data_pdu_type)
 {
-  int len;
+  int len = 0;
 
   DEBUG(("in xrdp_rdp_send_data"));
   s_pop_layer(s, rdp_hdr);
@@ -298,7 +306,7 @@ xrdp_rdp_send_data(struct xrdp_rdp* self, struct stream* s,
 int APP_CC
 xrdp_rdp_send_data_update_sync(struct xrdp_rdp* self)
 {
-  struct stream* s;
+  struct stream * s = (struct stream *)NULL;
 
   make_stream(s);
   init_stream(s, 8192);
@@ -327,8 +335,8 @@ xrdp_rdp_send_data_update_sync(struct xrdp_rdp* self)
 static int APP_CC
 xrdp_rdp_parse_client_mcs_data(struct xrdp_rdp* self)
 {
-  struct stream* p;
-  int i;
+  struct stream* p = (struct stream *)NULL;
+  int i = 0;
 
   p = &(self->sec_layer->client_mcs_data);
   p->p = p->data;
@@ -358,6 +366,13 @@ xrdp_rdp_parse_client_mcs_data(struct xrdp_rdp* self)
       self->client_info.bpp = 24;
       break;
   }
+  if (self->client_info.max_bpp > 0)
+  {
+    if (self->client_info.bpp > self->client_info.max_bpp)
+    {
+      self->client_info.bpp = self->client_info.max_bpp;
+    }
+  }
   p->p = p->data;
   DEBUG(("client width %d, client height %d bpp %d",
          self->client_info.width, self->client_info.height,
@@ -428,7 +443,8 @@ xrdp_rdp_send_demand_active(struct xrdp_rdp* self)
   out_uint16_le(s, 0x200); /* Protocol version */
   out_uint16_le(s, 0); /* pad */
   out_uint16_le(s, 0); /* Compression types */
-  out_uint16_le(s, 0); /* pad use 0x40d for rdp packets, 0 for not */
+  //out_uint16_le(s, 0); /* pad use 0x40d for rdp packets, 0 for not */
+  out_uint16_le(s, 0x40d); /* pad use 0x40d for rdp packets, 0 for not */
   out_uint16_le(s, 0); /* Update capability */
   out_uint16_le(s, 0); /* Remote unshare capability */
   out_uint16_le(s, 0); /* Compression level */
@@ -487,10 +503,10 @@ xrdp_rdp_send_demand_active(struct xrdp_rdp* self)
   out_uint8(s, 0); /* multi dest blt */
   out_uint8(s, 0); /* multi pat blt */
   out_uint8(s, 0); /* multi screen blt */
-  out_uint8(s, 0); /* multi rect */
+  out_uint8(s, 1); /* multi rect */
   out_uint8(s, 0); /* fast index */
-  out_uint8(s, 0); /* polygon */
-  out_uint8(s, 0); /* polygon */
+  out_uint8(s, 0); /* polygonSC ([MS-RDPEGDI], 2.2.2.2.1.1.2.16) */
+  out_uint8(s, 0); /* polygonCB ([MS-RDPEGDI], 2.2.2.2.1.1.2.17) */
   out_uint8(s, 0); /* polyline */
   out_uint8(s, 0); /* unused */
   out_uint8(s, 0); /* fast glyph */
@@ -644,8 +660,8 @@ static int APP_CC
 xrdp_process_capset_bmpcache2(struct xrdp_rdp* self, struct stream* s,
                               int len)
 {
-  int Bpp;
-  int i;
+  int Bpp = 0;
+  int i = 0;
 
   self->client_info.bitmap_cache_version = 2;
   Bpp = (self->client_info.bpp + 7) / 8;
